<?php
/*
Prestashop Verotel FlexPay Payment Module
Copyright (C) 2013 Netsafe Solutions s.r.o..

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details. You should have received a copy of the
GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

if (!defined('_PS_VERSION_'))
    exit;

class verotel_flexpay extends PaymentModule {

    public function __construct() {
        $this->name = 'verotel_flexpay';
        $this->tab = 'payments_gateways';
        $this->version = '1.0';
        $this->author = 'Netsafe Solutions s.r.o.';
        $this->module_key = 'fc5e57245c78ce68d0ba38799915bd76';
		
        $this->currencies = true;
		$this->currencies_mode = 'checkbox';
        $this->supported_currencies = array(
            'USD' => 1,
            'EUR' => 1,
            'GBP' => 1,
            'AUD' => 1,
            'CAD' => 1,
            'CHF' => 1,
            'DKK' => 1,
            'NOK' => 1,
            'SEK' => 1
        );

        $this->need_instance = 1;
        $this->ps_versions_compliancy = array('min' => '1.5', 'max' => '1.6'); 

        parent::__construct(); 

        $this->displayName = $this->l('Verotel FlexPay');
        $this->description = $this->l('Process transactions through the Verotel FlexPay.');
        $this->confirmUninstall = $this->l('Are you sure?');

        if (!Configuration::get('VEROTEL_FLEXPAY_SHOP_ID'))  
            $this->warning = $this->l('No Verotel FlexPay shop id provided'); 
        if (!Configuration::get('VEROTEL_FLEXPAY_SIGNATURE'))  
            $this->warning = $this->l('No Verotel FlexPay signature provided');

        $this->check_currency_support();
    }

    private function check_currency_support() {
        $currencies = Currency::getPaymentCurrencies($this->id);

        $unsupported_currencies = array();
        foreach($currencies as $currency){
            if (! array_key_exists($currency['iso_code'], $this->supported_currencies)){
                array_push($unsupported_currencies, $currency['iso_code']);
            }
        }

        if (count($unsupported_currencies) > 0){
            $this->warning = 'Verotel FlexPay does not support selected payment currencies: ' . join(', ', $unsupported_currencies);
        }
    }

    public function install() {
        if (
            parent::install()
            && Configuration::updateValue('VEROTEL_FLEXPAY_SHOP_ID', '')
            && Configuration::updateValue('VEROTEL_FLEXPAY_SIGNATURE', '')
            && $this->registerHook('payment')
            && $this->registerHook('paymentReturn')
        ) {
            return true;
        } else {
            return false;
        }
    }

    public function uninstall() {
        if (
            Configuration::deleteByName('VEROTEL_FLEXPAY_SHOP_ID')
            && Configuration::deleteByName('VEROTEL_FLEXPAY_SIGNATURE')
            && parent::uninstall()
        ) {
            return true;
        } else {
            return false;
        }
    }

    public function getContent() {
        if (Tools::isSubmit($this->name.'_SUBMIT')) {
            $this->_html .= $this->updateConfiguration();
        }

        $this->_html .= $this->displayForm();
        $this->_html .= $this->displaySetupInstructions();
        return $this->_html;
    }

    private function displaySetupInstructions() {
        $html = '';

        $this->check_currency_support();

        if ($this->warning) {
            $html .= '<div class="warn"><ul><li>' . $this->warning . '</li></ul></div>';
        }

        $html .= '<p><fieldset><legend>' . $this->l('Setup instruction') . '</legend>' .
            '<p>' .
            $this->l('You have to <strong>enable FlexPay</strong> ' .
            'in the Verotel ControlCenter FlexPay Options.') .
            '<br />' . $this->l('You mustn\'t forget to fill in there following URLs:') .
            '<ul>' .
                '<li><strong>' . $this->l('Postback script URL') . ':</strong> ' . 
                    $this->get_callback_url() .
                '</li>' .
                '<li><strong>' . $this->l('Success URL') . ':</strong> ' .
                    $this->get_success_url() .
                '</li>' .
            '</ul>' .
            '</p>' .
            '</fieldset></p>';

        return $html;
    }

    private function displayForm() {
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
         
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings')
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Verotel Shop ID'),
                    'name' => 'VEROTEL_FLEXPAY_SHOP_ID',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Signature Key'),
                    'name' => 'VEROTEL_FLEXPAY_SIGNATURE',
                    'size' => 50,
                    'required' => true
                )
            ),
            'submit' => array(
                'title' => $this->l('Update settings'),
                'class' => 'button'
            )
        );
         
        $helper = new HelperForm();
         
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
         
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
         
        $helper->title = $this->displayName; 
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = $this->name.'_SUBMIT';
        $helper->toolbar_btn = array(
            'save' =>
            array(
                'desc' => $this->l('Update settings'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );
         
        $helper->fields_value['VEROTEL_FLEXPAY_SHOP_ID'] = Configuration::get('VEROTEL_FLEXPAY_SHOP_ID');
        $helper->fields_value['VEROTEL_FLEXPAY_SIGNATURE'] = Configuration::get('VEROTEL_FLEXPAY_SIGNATURE');
         
        return $helper->generateForm($fields_form);


        return '<form action="' . $_SERVER['REQUEST_URI'] . '" method="post">' .
            '<fieldset><legend>' . $this->l('Configuration') . '</legend>' . 
            '<label>' . $this->l('Verotel Shop ID') . ':</label><div class="margin-form"><input name="VEROTEL_FLEXPAY_SHOP_ID" type="text" value="' .
                Configuration::get('VEROTEL_FLEXPAY_SHOP_ID') . '" />' .
                '<p>' . $this->l('Verotel Shop (website) ID') . '</p>' .
                '</div>' .
            '<label>' . $this->l('Signature Key') . ':</label><div class="margin-form"><input name="VEROTEL_FLEXPAY_SIGNATURE" type="text" value="' .
                Configuration::get('VEROTEL_FLEXPAY_SIGNATURE') . '" />' .
                '<p>' . $this->l('You will find it in Verotel Control Center FlexPay Options.') . '</p>' .
                '</div>' .
            '<input type="submit" name="verotel_flexpay_SUBMIT" id="verotel_flexpay_SUBMIT" value="' .
                $this->l('Update settings') . '" />' .
            '</fieldset></form>';
    }

    private function updateConfiguration() {
        $errors = array();

        if (!Tools::getValue('VEROTEL_FLEXPAY_SHOP_ID')) {
            $errors []= $this->displayError(
                $this->l('The Verotel Shop ID field can\'t be left blank. Please check the correct value with Verotel if you\'re unsure.'));
        }
        
        if (!Tools::getValue('VEROTEL_FLEXPAY_SIGNATURE')) {
            $errors []= $this->displayError(
                $this->l('The Signature Key field can\'t be left blank. Please check the correct value with Verotel if you\'re unsure.'));
        }

        if (count($errors) > 0){
            return join($errors);
        }

        Configuration::updateValue('VEROTEL_FLEXPAY_SHOP_ID', trim(Tools::getValue('VEROTEL_FLEXPAY_SHOP_ID')));
        Configuration::updateValue('VEROTEL_FLEXPAY_SIGNATURE', trim(Tools::getValue('VEROTEL_FLEXPAY_SIGNATURE')));
        return $this->displayConfirmation($this->l('Changes have all been saved'));
    }

    public function hookPayment($params) {

        $cart = $params['cart'];
        $customer = new Customer(intval($cart->id_customer));
        $shop = new Shop(intval($cart->id_shop));
        $currency = new Currency($cart->id_currency);
		$total = (float)$cart->getOrderTotal(true, Cart::BOTH);        

        $flexpay_url = FlexPay::get_purchase_URL(
            Configuration::get('VEROTEL_FLEXPAY_SIGNATURE'),
            array(
                'shopID' => Configuration::get('VEROTEL_FLEXPAY_SHOP_ID'),
                'priceAmount' => $total,
                'priceCurrency' => $currency->iso_code,
                'referenceID' => $this->get_reference_id_from_cart($cart),
                'description' => $this->l('Your order from') . ' ' . $shop->name,
                'custom1' => $cart->id,
                'custom2' => $cart->secure_key,
                'custom3' => $cart->id_shop,
                'email' => $customer->email,
                'origin' => 'PrestaShop',
            )
        );
        global $smarty;
        $smarty->assign(array('flexpay_url' => $flexpay_url));

        return $this->display(__FILE__, 'pay.tpl');
    }

    public function approve_cart($cart, $amount, $currencyIso, $secure_key) {
        $id_currency = Currency::getIdByIsoCode($currencyIso, $cart->id_shop);

        $id_order = Order::getOrderByCartId($cart->id);
        
        if (!$id_order){
            $message = 'Verotel FlexPay Reference ID='
                        .$this->get_reference_id_from_cart($cart);
            $this->validateOrder(
                $cart->id, Configuration::get('PS_OS_PAYMENT'), $amount,
                $this->name, $message, array(), $id_currency, false, $secure_key
            );
        }else{
            $order = new Order($id_order);
            $order->setCurrentState(Configuration::get('PS_OS_PAYMENT'));
        }
    }

    public function mark_cart_as_error($cart, $amount, $currencyIso, $secure_key, $message) {
        $id_currency = Currency::getIdByIsoCode($currencyIso, $cart->id_shop);

        $message_with_details = $message.' Verotel FlexPay Reference ID='
                    .$this->get_reference_id_from_cart($cart);

        $this->validateOrder(
            $cart->id, Configuration::get('PS_OS_ERROR'), $amount,
            $this->name, $message_with_details, array(), $id_currency, false, $secure_key
        );
    }

    private function get_reference_id_from_cart($cart) {
        return ($cart->id . ':' .
            $cart->id_customer . ':' .
            $cart->id_shop
        );
    }

    private function get_callback_url() {
        return $this->get_postback_url_for_action( 'Callback' );
    }

    private function get_success_url() {
        return $this->get_postback_url_for_action( 'Success' );
    }

    private function get_postback_url_for_action( $action ) {
        return $this->strip_lang_id(
            $this->context->link->getModuleLink(
                $this->name,
                $action,
                array(),
                true
            )
        );
    }

    private function strip_lang_id($url) {
        $no_lang_url = preg_replace('/[&\?]?id_lang=\d+/i', '', $url);

		$id_lang = $this->context->language->id;
        $lang_uri_part = Language::getIsoById($id_lang);

        $no_lang_url = preg_replace('/\\/'.$lang_uri_part.'\\//i', '/', $no_lang_url);

        return $no_lang_url;
    }
}

include("FlexPay.php");
